#include "tmEventSetup/esAlgorithmHandle.hh"

#include "tmTable/tmTable.hh"

#include "tmUtil/tmUtil.hh"

#include <boost/lexical_cast.hpp>

namespace tmeventsetup
{

// keys for tables
const std::string kName("name");
const std::string kIndex("index");
const std::string kModuleId("module_id");
const std::string kModuleIndex("module_index");
const std::string kExpression("expression");
const std::string kLabels("labels");


void
esAlgorithmHandle::replace(const std::string& from, const std::string& to)
{
  size_t p = 0;
  while ((p = expression_in_condition_.find(from, p)) != std::string::npos)
  {
    bool replace = true;
    const size_t index = p + from.length();
    if (index < expression_in_condition_.size())
    {
      switch (expression_in_condition_.at(index))
      {
        case ' ': break;
        case ')': break;
        default: replace = false;
      }
    }
    if (replace) expression_in_condition_.replace(p, from.length(), to);
    p += to.length();
    if (p >= expression_in_condition_.size()) break;
  }
}


void
esAlgorithmHandle::init(const tmtable::Row& algorithm)
{
  name_ = algorithm.at(kName);
  expression_ = algorithm.at(kExpression);

  index_ = boost::lexical_cast<unsigned int>(algorithm.at(kIndex));
  module_id_ = boost::lexical_cast<unsigned int>(algorithm.at(kModuleId));
  module_index_ = boost::lexical_cast<unsigned int>(algorithm.at(kModuleIndex));

#if defined(SWIG)
  if (algorithm.count(kLabels))
  {
    setLabels(tmutil::splitLabels(algorithm.at(kLabels)));
  }
#endif
}


void
esAlgorithmHandle::setExpressionInCondition(const std::map<std::string, std::string>& map)
{
  TM_LOG_DBG("");
  expression_in_condition_ = expression_;
  for (const auto& value: map)
  {
    TM_LOG_DBG(TM_QUOTE(value.first) << " : " << TM_QUOTE(value.second));
    replace(value.first, value.second);
  }
}


#if defined(SWIG)
void
esAlgorithmHandle::setLabels(const std::set<std::string>& labels)
{
  labels_ = labels;
}
#endif

} // namespace tmeventsetup
